#include "grbc/ext_cmake.h"
#include "grbc/ext.h"
#include "grbc/helpers.h"
#include "grbc/spec.h"
#include "sol/raii.hpp"
#include <cstdlib>
#include <filesystem>

CMakeProject EXT_grbc_import_cmake(const std::string &cmake_path,
                                   const CMakeConfig &cfg) {
  CMakeProject project{};

  log_msg(("configuring cmake project: " + cmake_path).c_str());

  if (!std::filesystem::exists(cmake_path)) {
    grbc_exception("CMake cannot configure in non-existant directory: " +
                   cmake_path);
  }

  if (!std::filesystem::exists(cmake_path + "/CMakeLists.txt")) {
    grbc_exception("No CMakeLists.txt file in: " + cmake_path);
  }

  std::filesystem::path old_path = std::filesystem::current_path();

  // Change directory into the cmake dir
  std::filesystem::current_path(cmake_path);

  std::filesystem::create_directory("grbc_configure");

  std::filesystem::current_path("./grbc_configure");

  // Create configure arg string
  std::string configure_arguments;

  for (const std::string &argument : cfg.configure_arguments) {
    configure_arguments += argument + " ";
  }

  // Run configure
  int exit_code =
      std::system(("cmake -S .. -B . -GNinja " + configure_arguments).c_str());

  if (exit_code != EXIT_SUCCESS)
    grbc_exception("Failed to configure cmake project in: " + cmake_path);

  std::filesystem::current_path(old_path);

  return project;
}

Package EXT_grbc_get_cmake_library(const CMakeProject &self,
                                   const std::string &library_name) {
  std::filesystem::path library_path = library_name;
}

void grbc_cmake_init(sol::state &lua) {
  lua.new_usertype<CMakeConfig>(
      "CMakeConfig", sol::constructors<CMakeConfig(sol::table)>(),
      "configure_arguments", &CMakeConfig::configure_arguments);
  lua.new_usertype<CMakeProject>("CMakeProject", "source_dir",
                                 &CMakeProject::source_dir, "build_dir",
                                 &CMakeProject::build_dir);

  lua["CMakeProject"]["get_library"] = EXT_grbc_get_cmake_library;

  lua.set("grbc_import_cmake", EXT_grbc_import_cmake);
}

Extension grbc_cmake() {
  Extension ext{};
  ext.name = GRBC_EXT_cmake_NAME;
  ext.hook_init = grbc_cmake_init;

  return ext;
}