#include "grbc/ext_profiles.h"
#include "grbc/ext.h"
#include "grbc/helpers.h"
#include "grbc/spec.h"
#include "grbc/state.h"

std::vector<Profile> &EXT_profiles_get_profiles() {
  static std::vector<Profile> profiles{};

  return profiles;
}

Profile *EXT_profiles_get_current_profile() {
  static Profile profile{};

  return &profile;
}

void EXT_profiles_set(const std::string &profile_id) {
  for (auto &profile : EXT_profiles_get_profiles()) {
    if (profile.name != profile_id)
      continue;

    *EXT_profiles_get_current_profile() = profile;

    log_msg(("set current profile to: " + profile_id).c_str());

    // Maybe warn the user about this?
    GState::get().global_compiler_flags = " ";
    GState::get().global_linker_flags = " ";

    for (auto &linker_flag : profile.linker_flags) {
      GState::get().global_linker_flags += linker_flag + " ";
    }

    for (auto &compiler_flag : profile.compiler_flags) {
      GState::get().global_compiler_flags += compiler_flag + " ";
    }

    return;
  }

  grbc_exception("Failed to locate profile with name: " + profile_id);
}

void grbc_profiles_init(sol::state &lua) {
  // Set default profiles
  Profile debug_profile{};
  debug_profile.name = "debug";
  debug_profile.compiler_flags = {"-g", "-O1"};

  Profile release_with_debug_symbols_profile{};
  release_with_debug_symbols_profile.name = "release_with_debug_symbols";
  release_with_debug_symbols_profile.compiler_flags = {"-g", "-O3", "-DNDEBUG"};

  Profile release_profile{};
  release_profile.name = "release";
  release_profile.linker_flags = {"-O3", "-DNDEBUG"};

  EXT_profiles_get_profiles().push_back(debug_profile);
  EXT_profiles_get_profiles().push_back(release_with_debug_symbols_profile);
  EXT_profiles_get_profiles().push_back(release_profile);

  if (!grbc_has_option("compiler_profile")) {
    EXT_profiles_set("debug"); // Always use debug as the default profile
  } else {
    EXT_profiles_set(grbc_get_option("compiler_profile"));
  }

  lua.set("grbc_set_profile", EXT_profiles_set);

  log_msg("profile support loaded!");
}

Extension grbc_profiles() {
  Extension ext{};
  ext.name = GRBC_EXT_profiles_NAME;
  ext.hook_init = grbc_profiles_init;

  return ext;
}